package my;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Map;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.emf.edapt.internal.migration.execution.internal.ClassLoaderFacade;
import org.eclipse.emf.edapt.migration.MigrationException;
import org.eclipse.emf.edapt.migration.ReleaseUtils;
import org.eclipse.emf.edapt.migration.execution.Migrator;
import org.eclipse.emf.edapt.migration.execution.MigratorRegistry;
import org.eclipse.emf.edapt.spi.history.Release;
import org.eclipse.uml2.uml.internal.resource.UMLResourceFactoryImpl;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import my2Ecore.ClassB;
import my2Ecore.My2EcoreFactory;
import myEcore.ClassA;
import myEcore.MyEcoreFactory;

public class MyEcoreTest {

	/**
	 * Perform some initialization tasks
	 * 
	 * @throws MigrationException
	 */
	@BeforeClass
	public static void init() throws MigrationException {
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> m = reg.getExtensionToFactoryMap();
		m.put("xmi", new XMIResourceFactoryImpl());
		m.put("uml", new UMLResourceFactoryImpl());

		// Register the migrator
		File file = new File("model/My.history");
		MigratorRegistry.getInstance().registerMigrator(URI.createFileURI(file.getAbsolutePath()),
				new ClassLoaderFacade(MyEcoreTest.class.getClassLoader()));

	}

	private void createV1_0_Model() {

		ClassA classA = MyEcoreFactory.eINSTANCE.createClassA();

		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> m = reg.getExtensionToFactoryMap();
		m.put("xmi", new XMIResourceFactoryImpl());

		ResourceSet rs = new ResourceSetImpl();
		Resource testResource = rs.createResource(URI.createURI("modelData/v1_0/testA.xmi"));

		testResource.getContents().add(classA);

		try {
			testResource.save(Collections.EMPTY_MAP);
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	private void createV1_1_Model() {

		ClassA classA = MyEcoreFactory.eINSTANCE.createClassA();
		classA.setAttV1_1("V1.1");
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> m = reg.getExtensionToFactoryMap();
		m.put("xmi", new XMIResourceFactoryImpl());

		ResourceSet rs = new ResourceSetImpl();
		Resource testResource = rs.createResource(URI.createURI("modelData/v1_1/testA.xmi"));

		testResource.getContents().add(classA);

		try {
			testResource.save(Collections.EMPTY_MAP);
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	private void createV2_0_Model() {

		ClassA classA = MyEcoreFactory.eINSTANCE.createClassA();
		ClassB classB = My2EcoreFactory.eINSTANCE.createClassB();

		classA.setClassB(classB);

		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> m = reg.getExtensionToFactoryMap();
		m.put("xmi", new XMIResourceFactoryImpl());
		m.put("uml", new UMLResourceFactoryImpl());

		ResourceSet rs = new ResourceSetImpl();
		Resource testResource = rs.createResource(URI.createURI("modelData/v2/testA.xmi"));
		Resource testBResource = rs.createResource(URI.createURI("modelData/v2/testB.xmi"));

		testResource.getContents().add(classA);
		testBResource.getContents().add(classB);

		try {
			testResource.save(Collections.EMPTY_MAP);
			testBResource.save(Collections.EMPTY_MAP);
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	@Test
	public void loadV1_0_Test() throws IOException, MigrationException {
		Assert.assertNotNull(loadResource("modelData/v1_0/testA.xmi"));
	}

	@Test
	public void loadV1_1_Test() throws IOException, MigrationException {
		Assert.assertNotNull(loadResource("modelData/v1_1/testA.xmi"));
	}

	@Test
	public void loadV2_0_Test() throws IOException, MigrationException {
		Assert.assertNotNull(loadResource("modelData/v2_0/testA.xmi"));
	}

	public ResourceSet loadResource(String path) throws IOException, MigrationException {

		URI uri = URI.createURI(path);
		String nsURI = ReleaseUtils.getNamespaceURI(uri);
		
		Migrator migrator = MigratorRegistry.getInstance().getMigrator(nsURI);

		if (migrator != null) {
			final Release release = migrator.getRelease(uri).iterator().next();
			if (!release.isLatestRelease()) {
				ResourceSet migratedResources = performMigration(migrator, uri, release);
				return migratedResources;
			} else {
				ResourceSet resourceSet = new ResourceSetImpl();
				Resource resource = resourceSet.getResource(URI.createURI(path), true);
				EcoreUtil.resolveAll(resource);
				return resourceSet;
			}

		}

		return null;
	}

	private ResourceSet performMigration(final Migrator migrator, URI uri, final Release release)
			throws IOException {
		try {

			ResourceSet resourceSet = migrator.migrateAndLoad(Collections.singletonList(uri), release, null,
					new NullProgressMonitor());
			return resourceSet;

		} catch (MigrationException e) {
			e.printStackTrace();
			throw new IOException("Cannot migrate resources ", e);
		}

	}

	/**
	 * Main class to create some test data
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		MyEcoreTest met = new MyEcoreTest();
		// met.init();
		// met.createV1_0_Model();
		// met.createV1_1_Model();
		met.createV2_0_Model();
	}
}
